## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Groups section
# @details This class handles a collection of all functions that are listed in the Groups section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Groups:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief List groups
	# @details Calls the API to list all groups this user is member in
	# @return Either a @p dict in json format or a @p list 
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def get_groups(self) -> Union[dict, list]:
		
		r = requests.get(f"{self.address}:{self.port}/v1/groups/{self.number}", headers={'Content-Type': 'application/json'})
		
		r = r.json()
		return r
	
	## @brief Create group
	# @details Calls the API to create a new group
	# @param	name				The name of the group
	# @param	description			The description of the group
	# @param	members				The members to be added to the group
	# @param	group_link			<b>(Optional)</b>Defines if a group link should be generated, this doesn't work correctly (2022-12-18). This argument defaults to @p disabled.
	# @param	perm_add_members	<b>(Optional)</b>Defines who is allowed to add new members. This argument defaults to @p only-admins.
	# @param	perm_edit_group		<b>(Optional)</b>Defines who is allowed to edit the group. This argument defaults to @p only-admins.
	# @return A @p dict in json format
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def create_group(
		self,
		name: str,
		description: str,
		members: list[str],
		group_link: str="disabled",
		perm_add_members: str="only-admins",
		perm_edit_group: str="only-admins"		
	) -> dict:
		r = requests.post(f"{self.address}:{self.port}/v1/groups/{self.number}", headers={'Content-Type': 'application/json'}, json={"description": description, "group_link": create_group, "members": members, "name": name, "permissions": {"add_members": perm_add_members, "edit_group": perm_edit_group}})
		
		r = r.json()
		return r
	
	## @brief Get group
	# @details Calls the API to list informations about a group
	# @param	groupid	The group ID
	# @return A @p dict in json format
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def get_group(self, groupid: str) -> dict:
		
		r = requests.get(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}", headers={'Content-Type': 'application/json'})
		
		r = r.json()
		return r
	
	## @brief Delete group
	# @details Calls the API to delete a group
	# @param	groupid	The group ID
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def delete_group(self, groupid):
		
		r = requests.delete(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Add admins
	# @details Calls the API to add admins
	# @param	groupid	The group ID
	# @param	admins	The members that should be made an admin
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def add_admins(self, groupid: str, admins: list[str]) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}/admins", headers={'Content-Type': 'application/json'}, json={"admins": admins})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Remove admins
	# @details Calls the API to remove admins
	# @param	groupid	The group ID
	# @param	admins	The admins that should be degraded to members
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def remove_admins(self, groupid: str, admins: list[str]) -> Union[dict, str]:
		payload = {'admins': admins}
		
		r = requests.delete(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}/admins", headers={'Content-Type': 'application/json'}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Block group
	# @details Calls the API to block a group
	# @param	groupid	The group ID
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def block_group(self, groupid: str) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/groups/{self.number}/{self.groupid}/block", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Join group
	# @details Calls the API to join a group
	# @param	groupid	The group ID
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def join_group(self, groupid: str) -> Union[list, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}/join", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Add members to group
	# @details Calls the API to add members to a group
	# @param	groupid	The group ID
	# @param	members	The members to be added to the group
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def add_members(self, groupid: str, members: list[str]) -> Union[list, str]:
		 
		r = requests.post(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}/members", headers={'Content-Type': 'application/json'}, json={"members": members})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Remove members from group
	# @details Calls the API to remove members from a group
	# @param	groupid	The group ID
	# @param	members	The members to be removed from the group
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def remove_members(self, groupid: str, members: list[str]) -> Union[list, str]:
		payload = {'members': members}
		
		r = requests.delete(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}/members", headers={'Content-Type': 'application/json'}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Quit group
	# @details Calls the API to quit a group
	# @param	groupid	The group ID
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18	
	def quit_group(self, groupid: str) -> Union[list, str]:
		r = requests.post(f"{self.address}:{self.port}/v1/groups/{self.number}/{groupid}/quit", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
		
		



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Profiles section
# @details This class handles a collection of all functions that are listed in the Profiles section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Profiles:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief Update profile
	# @details Calls the API to update the profile
	# @param	**name		<b>(Optional)</b> The new name of the user associated with the number in the config. 
	# As this is a keyword argument, it has to be specified with <tt>name=\<name\></tt>, the data type is @p str. <b>One of the both arguments is necessary, two are optional!</b>
	# @param	**avatar	<b>(Optional)</b> The new avatar of the user associated with the number in the config as base64 string.
	# As this is a keyword argument, it has to be specified with <tt>avatar=\<avatar\></tt>, the data type is @p str. <b>One of the both arguments is necessary, two are optional!</b>
	# @return Either a @p dict in json format or a @p str. 
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18	
	def update_profile(self, **kwargs) -> Union[dict, str]:
		for key, value in kwargs.items():
			name = None
			avatar = None
			if key == "name":
				name = value
			elif key == "avatar":
				avatar = value
		if name == None and avatar == None:
			return {"error": "No changes to be made"}
		elif name != None and avatar == None:
			payload = {"name": name}
		elif name == None and avatar != None:
			payload = {"base64_avatar": avatar}
		else:
			payload = {"base64_avatar": avatar, "name": name}
		
		r = requests.put(f"{self.address}:{self.port}/v1/profiles/{self.number}", headers={"Content-Type": "application/json"}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
			



from .General import General
from .Devices import Devices
from .Groups import Groups
from .Messages import Messages
from .Attachments import Attachments
from .Profiles import Profiles
from .Identities import Identities
from .Reactions import Reactions
from .Search import Search
from .Contacts import Contacts
from .Client import Client

__all__ = ['General', 'Client', 'Devices', 'Groups', 'Messages', 'Attachments', 'Profiles', 'Identities', 'Reactions', 'Search', 'Contacts']



## @package signal_cli

import requests, json, os, pathlib, filetype, base64
from typing import Union

## @brief Class holding the collection of functions from the Attachments section
# @details This class handles a collection of all functions that are listed in the Attachments section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Attachments:

	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
		
	## @brief List attachments
	# @details Calls the API to list the existing attachments
	# @return Either a @p list or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def list_attachments(self) -> Union[list, str]:
		
		r = requests.get(f"{self.address}:{self.port}/v1/attachments", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Serve attachments
	# @details Calls the api to get a specified attachment as file, writes it to disc, guesses and adds the file extension, encodes it to base64, deletes the file and returns the base64 string
	# @param	attachment	The id of the attachment to serve
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def serve_attachment(self, attachment: str) -> Union[dict, str]:

		r = requests.get(f"{self.address}:{self.port}/v1/attachments/{attachment}", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			img = attachment
			with open(img, 'wb') as f:
				for chunk in r:
					f.write(chunk)
			fileinfo = filetype.guess(img)
			try:
				fullimg = img + "." + fileinfo.extension
			except:
				fullimg = img + ".bin"
			os.rename(img, fullimg)
			with open(fullimg, 'rb') as f:
				b64string = base64.b64encode(f.read()).decode('utf-8')
				ext = fullimg.split('.')[-1]
				r = f'data:image/{ext};base64,{b64string}'
			os.remove(fullimg)
		return r
	
	## @brief Delete attachments
	# @details Calls the api to delete a specified attachment
	# @param	attachment	The id of the attachment to serve
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def delete_attachment(self, attachment: str) -> Union[dict, str]:
		
		r = requests.delete(f"{self.address}:{self.port}/v1/attachments/{attachment}", headers={'Content-Type': 'application/json'})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Contacts section
# @details This class handles a collection of all functions that are listed in the Contacts section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Contacts:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
		
	## @brief Update contact
	# @details Calls the API to update the details of an existing contact or, if not existing, creating a new contact with the given information
	# @param	expiration	The time for messages to self-destruct, set 0 for never self-destruct
	# @param	name		The name of the contact
	# @param	recipient	The phone number of the contact, in international format
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def update_contact(self, expiration: int, name: str, recipient: str) -> Union[dict, str]:
		
		payload = {"expiration_in_seconds": expiration, "name": name, "recipient": recipient}
		r = requests.put(f"{self.address}:{self.port}/v1/contacts/{self.number}", headers={"Content-Type": "application/json"}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Identities section
# @details This class handles a collection of all functions that are listed in the Identities section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Identities:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief List indentities
	# @details Calls the API to list all identities of known numbers
	# @return A @p list with dicts
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18	
	def list_identites(self) -> list[dict]:
		r = requests.get(f"{self.address}:{self.port}/v1/identities/{self.number}", headers={"Content-Type": "application/json"})
		
		r = r.json()
		return r
	
	## brief Trust identity
	# @details Calls the API to trust an identity
	# @param	number_to_trust			The number that should be trusted
	# @param	safety_number			The safety number between this and the to be trusted account
	# @param	trust_all_known_keys	<b>(Optional)</b> This sets if all known keys should be trusted. This argument defaults to @p False.
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def trust_identity(self, number_to_trust: str, safety_number: str, trust_all_known_keys: bool=False) -> Union[dict, str]:
		payload = {"trust_all_known_keys": trust_all_known_keys, "verified_safety_number": safety_number}
		
		r = requests.put(f"{self.address}:{self.port}/v1/identities/{self.number}/trust/{number_to_trust}", headers={"Content-Type": "application/json"}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	



## @package signal_cli

import requests, json, os, pathlib, base64, filetype
from typing import Union

## @brief signal-cli client object
# @details A client object building an instance of the module, containing the configuration for the session
# @param	address	The address where the signal-cli-rest-api is accessible
# @param	port	The port where the signal-cli-rest-api is accessible
# @param	number	<b>(Optional)</b> The phone number to be used for this object, 
# has to be registered in the signal-cli-rest-api. This argument defaults to "+441189998819991197253"
# @author Felix Hune
# @version 1.0
# @date 2022-12-25
class Client:
	
	def __init__(self, address: str, port: int, number: str="+441189998819991197253"):
		self.address = address
		self.port = port
		self.number = number
	
	## @brief get address
	# @return A @p str containing the address
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-25
	def get_address(self) -> str:
		return self.address
	
	## @brief set address
	# @param	address	The address to set
	# @return @p None
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-25
	def set_address(self, address: str):
		self.address = address
	
	## @brief get port
	# @return A @p int containing the port
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-25
	def get_port(self) -> int:
		return self.port
	
	## @brief set port
	# @param	port	The port to set
	# @return @p None
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-25
	def set_port(self, port: int):
		self.port = port
	
	## @brief get number
	# @return A @p str containing the number
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-25
	def get_number(self) -> str:
		return self.number
	
	## @brief set number
	# @param	number	The number to set
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-25
	def set_number(self, number: str):
		self.number = number
	



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Search section
# @details This class handles a collection of all functions that are listed in the Search section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Search:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief Search numbers
	# @details Calls the API checking if the given numbers are registered with the Signal service or not. This isn't working correctly (2022-12-18)
	# @param	numbers	List of phone numbers to check
	# @return Either a @list with @dicts or a @dict
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def search(self, numbers: list[str]) -> Union[list[dict],dict]:
		
		r = requests.get(f"{self.address}:{self.port}/v1/search", headers={"Content-Type": "application/json"}, params={"numbers": numbers})
		
		r = r.json()
		return r
	



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Messages section
# @details This class handles a collection of all functions that are listed in the Messages section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Messages:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief Receive messages
	# @details Calls the API to list all new incoming messages
	# @param	timeout	<b>(Optional)</b> Sets the timeout parameter. This argument defaults to @p 1.
	# @return Either a @p list with @p dicts or a @p dict
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def receive(self, timeout: int=1) -> Union[list[dict], dict]:
		
		r = requests.get(f"{self.address}:{self.port}/v1/receive/{self.number}", headers={'Content-Type': 'application/json'}, params={"timeout": str(timeout)})
		r = r.json()
		return r
	
	## @brief Show typing indicator
	# @details Calls the API to show the typing indicator in a specified conversation
	# @param	recipient	The phone number of the recipient, maybe also group id?
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def show_typing(self, recipient: str) -> Union[dict, str]:
		
		payload = {"recipient": recipient}
		r = requests.put(f"{self.address}:{self.port}/v1/typing-indicator/{self.number}", headers={'Content-Type': 'application/json'}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Hide typing indicator
	# @details Calls the API to hide the typing indicator in a specified conversation
	# @param	recipient	The phone number of the recipient, maybe also group id?
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18	
	def hide_typing(self, recipient: str) -> Union[dict, str]:
		
		payload = {"recipient": recipient}
		r = requests.delete(f"{self.address}:{self.port}/v1/typing-indicator/{self.number}", headers={'Content-Type': 'application/json'}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Send message
	# @details Calls the API to send a message
	# @param	message		The message text
	# @param	recipients	The list of recipients to send the message to
	# @param	attachments	<b>(Optional)</b> A list of attachments encoded to base64. This argument defaults to <tt>[]</tt>
	# @return A @p dict in json format
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def send(self, message: str, recipients: list[str], attachments: list[str]=[]) -> dict:
		
		r = requests.post(f"{self.address}:{self.port}/v2/send", headers={'Content-Type': 'application/json'}, json={"base64_attachments": attachments, "message": message, "number": self.number, "recipients": recipients})
		
		r = r.json()
		return r
		



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the General section
# @details This class handles a collection of all functions that are listed in the General section in the documentation of the signal-cli-rest-api.
# There is no function for /v1/health because it's just an internal request for the docker container
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class General:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief List API information
	# @details Calls the API to get informations about the API
	# @return A @p dict in json format
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def about(self) -> dict:
		
		r = requests.get(f"{self.address}:{self.port}/v1/about", headers={'Content-Type': 'application/json'})
		return r.json()
		
	## @brief List global configuration
	# @details Calls the API to get the current global configuration of the API
	# @return A @p dict in json format
	# @author Felix Hunne
	# @version 1.0
	# @date 2022-12-18
	def get_configuration(self) -> dict:
		
		r = requests.get(f"{self.address}:{self.port}/v1/configuration", headers={'Content-Type': 'application/json'})
		
		r = r.json()
		return r
	
	## @brief Set global configuration
	# @details Calls the API to set the global configuration of the API
	# @param	data	The data that should be set as global configuration, has to be a @p dict in json format
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def set_configuration(self, data: dict) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/configuration", headers={'Content-Type': 'application/json'}, json=data)
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief List number configuration
	# @details Calls the API to get the current number configuration
	# @return A @p dict in json format
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def get_number_configuration(self) -> dict:
		
		r = requests.get(f"{self.address}:{self.port}/v1/configuration/{self.number}/settings", headers={'Content-Type': 'application/json'})
		return r.json()
	
	## @brief Set number configuration
	# @details Calls the API to set the number configuration
	# @param	data	Teh data that should be set as number configuration, has to be a @p dict in json format
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def set_number_configuration(self, data: dict) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/configuration/{self.number}/settings", headers={'Content-Type': 'application/json'}, json=data)
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	
		
		



## @package signal_cli

import requests, json, os, pathlib
from typing import Union

## @brief Class holding the collection of functions from the Reactions section
# @details This class handles a collection of all functions that are listed in the Reactions section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Reactions:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief Send Reaction
	# @details Calls the API to send a reaction on a message in a chat
	# @param	reaction		The reaction to send. This has to be the Emoji as the Emoji character, not a unicode code
	# @param	recipient		The number of the recipient chat, maybe also a chat id?
	# @param	target_author	The number of the target author who wrote the message
	# @param	timestamp		The timestamp when the message was sent
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def send_reaction(self, reaction: str, recipient: str, target_author: str, timestamp: int) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/reactions/{self.number}", headers={"Content-Type": "application/json"}, json={"reaction": reaction, "recipient": recipient, "target_author": target_author, "timestamp": timestamp})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r

	## @brief Delete reaction
	# @details Calls the API to delete a reaction from a message in a chat
	# @param	reaction		The reaction to remove. This has to be the Emoji as the Emoji character, not a unicode code
	# @param	recipient		The number of the recipient chat, maybe also a chat id?
	# @param	target_author	The number of the target author who wrote the message
	# @param	timestamp		The timestamp when the message was sent
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def delete_reaction(self, reaction: str, recipient: str, target_author: str, timestamp: int) -> Union[dict, str]:
		
		payload = {"reaction": reaction, "recipient": recipient, "target_author": target_author, "timestamp": timestamp}
		r  = requests.delete(f"{self.address}:{self.port}/v1/reactions/{self.number}", headers={"Content-Type": "application/json"}, data=json.dumps(payload))
		
		try:
			r = r.json()
		except:
			r = r.text
		return r



## @package signal_cli

import requests, json, os, pathlib, base64
from typing import Union

## @brief Class holding the collection of functions from the Devices section
# @details This class handles a collection of all functions that are listed in the Devices section in the documentation of the signal-cli-rest-api
# @author Felix Hune
# @version 1.1
# @date 2022-12-25
class Devices:
	
	def __init__(self, Client):
		self.address = Client.get_address()
		self.port = Client.get_port()
		self.number = Client.get_number()
	
	## @brief Links another device
	# @details Calls the API to link another device to this device; this only works if this is the master device
	# @param	uri	The uri which is the content of the qr code that generates on the devices that should be linked when the linking is requested there
	# @return Either a @p dict in json format or a @str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def link_device(self, uri: str) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/devices/{self.number}", headers={'Content-Type': 'application/json'}, json={"uri": uri})
		try:
			r = r.json()
		except:
			r = r.text
		return r
	
	## @brief Request linking of this device
	# @details Calls the API to request a linking of this device to another device;
	# therefore the API returns a qr-code, which gets temporarily saved on the disc, 
	# then the function generates a base64 string from the picture, deletes the picture and returns the base64 string
	# @param	device_name	The name by which this device should show up on the other device
	# @return Either a @p dict in json format or a @str with the bas64
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def qrcodelink(self, device_name: str) -> Union[dict, str]:
		
		r = requests.get(f"{self.address}:{self.port}/v1/qrcodelink", headers={'Content-Type': 'application/json'}, params={"device_name": device_name})
		try:
			r = r.json()
		except:
			img = str(os.getpid()) + ".png"
			with open(img, 'wb') as f:
				for chunk in r:
					f.write(chunk)
			with open(img, 'rb') as f:
				b64string = base64.b64encode(f.read()).decode('utf-8')
				ext = img.split('.')[-1]
				r = f'data:image/{ext};base64,{b64string}'
			os.remove(img)
				
		return r

	## @brief Register new number
	# @details Calls the API to register a new account with the number in the config;
	# this only works if it isn't already registered. 
	# @pre Generate a captcha at https://signalcaptchas.org/registration/generate.html, then copy the link from the 'Open Signal'-button 
	# @param	captcha		The link from the button, obtainable as described in the preconditions
	# @param	use_voice	<b>(Optional)</b> Defines if the verification code should be send as SMS or as a phone call. This argument defaults to @p False
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def register(self, captcha: str, use_voice: bool=False) -> Union[dict, str]:
		captcha = captcha.replace("signalcaptcha://", "")
		r = requests.post(f"{self.address}:{self.port}/v1/register/{self.number}", headers={'Content-Type': 'application/json'}, json={"captcha": captcha, "use_voice": use_voice})
		try:
			r = r.json()
		except:
			r = r.text
		return r

	## @brief Verify registration
	# @details Calls the API to verify the registration of a new number
	# @pre Starting the registring process by calling @link register() @endlink
	# @param	token	The registration token obtained via the registration process message or call
	# @param	**pin	<b>(Optional)</b> The pin to secure the signal account. As this is a keyword argument, it has to be specified with <tt>pin=\<pin\></tt>, the data type is @p int
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def verify(self, token: str, **kwargs) -> Union[dict, str]:
		
		pin = None
		for key, value in kwargs.items():
			if key == "pin":
				pin = str(value)
		
		if pin == None:	
			r = requests.post(f"{self.address}:{self.port}/v1/register/{self.number}/verify/{token}", headers={'Content-Type': 'application/json'})
		else:
			r = requests.post(f"{self.address}:{self.port}/v1/register/{self.number}/verify/{token}", headers={'Content-Type': 'application/json'}, json={"pin": pin})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
		
	## @brief Unregister number
	# @details Calls the API to unregister the number in the config
	# @param	delete_account		<b>(Optional)</b> Defines if the account should be deleted from the Signal server. This argument defaults to @p False
	# @param	delete_local_data	<b>(Optional)</b> Defines if the local data for this account should be deleted. This arguments defaults to @p False
	# @return Either a @p dict in json format or a @p str
	# @author Felix Hune
	# @version 1.0
	# @date 2022-12-18
	def unregister(self, delete_account: bool=False, delete_local_data: bool=False) -> Union[dict, str]:
		
		r = requests.post(f"{self.address}:{self.port}/v1/unregister/{self.number}", headers={'Content-Type': 'application/json'}, json={"delete_account": delete_account, "delete_local_data": delete_local_data})
		
		try:
			r = r.json()
		except:
			r = r.text
		return r
			
